/* ***********************************************************
 * This file was automatically generated on 2012-07-02.      *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generator git on tinkerforge.com                   *
 *************************************************************/

package com.tinkerforge;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.concurrent.TimeUnit;


/**
 * Device for controlling up to 4 general purpose input/output pins
 */
public class BrickletIO4 extends Device {
	private final static byte FUNCTION_SET_VALUE = (byte)1;
	private final static byte FUNCTION_GET_VALUE = (byte)2;
	private final static byte FUNCTION_SET_CONFIGURATION = (byte)3;
	private final static byte FUNCTION_GET_CONFIGURATION = (byte)4;
	private final static byte FUNCTION_SET_DEBOUNCE_PERIOD = (byte)5;
	private final static byte FUNCTION_GET_DEBOUNCE_PERIOD = (byte)6;
	private final static byte FUNCTION_SET_INTERRUPT = (byte)7;
	private final static byte FUNCTION_GET_INTERRUPT = (byte)8;
	private final static byte CALLBACK_INTERRUPT = (byte)9;

	public class Configuration {
		public short directionMask;
		public short valueMask;

		public String toString() {
			return "[" + "directionMask = " + directionMask + ", " + "valueMask = " + valueMask + "]";
		}
	}

	/**
	 * This listener is triggered whenever a change of the voltage level is detected
	 * on pins where the interrupt was activated with :func:`SetPortInterrupt`.
	 * 
	 * The values are a bit mask that specifies which interrupts occurred
	 * and the current value bit mask of the port.
	 * 
	 * For example:
	 * 
	 * * (1, 1) means that an interrupt on pin 0 occurred and
	 *   currently pin 0 is high and pins 1-3 are low.
	 * * (9, 14) means that an interrupt on pins 0 and 3
	 *   occurred and currently pin 0 is low and pins 1-3 are high.
	 */
	public interface InterruptListener {
		public void interrupt(short interruptMask, short valueMask);
	}

	/**
	 * Creates an object with the unique device ID \c uid. This object can
	 * then be added to the IP connection.
	 */
	public BrickletIO4(String uid) {
		super(uid);

		expectedName = "IO-4 Bricklet";

		bindingVersion[0] = 1;
		bindingVersion[1] = 0;
		bindingVersion[2] = 0;

		callbacks[CALLBACK_INTERRUPT] = new CallbackListener() {
			public void callback(byte[] data) {
				ByteBuffer bb = ByteBuffer.wrap(data, 4, data.length - 4);
				bb.order(ByteOrder.LITTLE_ENDIAN);

				short interruptMask = IPConnection.unsignedByte(bb.get());
				short valueMask = IPConnection.unsignedByte(bb.get());

				((InterruptListener)listenerObjects[CALLBACK_INTERRUPT]).interrupt(interruptMask, valueMask);
			}
		};
	}

	/**
	 * Sets the output value (high or low) with a bit mask. The bit mask
	 * is 4 bit long, "true" refers to high and "false" refers to low.
	 * 
	 * For example: The value 0b0011 will turn the pins 0-1 high and the
	 * pins 2-3 low.
	 * 
	 *  \note
	 *   This function does nothing for pins that are configured as input.
	 *   Pull up resistors can be switched on with :func:`SetPortConfiguration`.
	 */
	public void setValue(short valueMask)  {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_SET_VALUE, (short)5);
		bb.put((byte)valueMask);

		ipcon.write(this, bb, FUNCTION_SET_VALUE, false);
	}

	/**
	 * Returns a bit mask of the values that are currently measured.
	 * This function works if the pin is configured to input
	 * as well as if it is configured to output.
	 */
	public short getValue() throws IPConnection.TimeoutException {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_GET_VALUE, (short)4);

		ipcon.write(this, bb, FUNCTION_GET_VALUE, true);

		byte[] response = null;
		try {
			response = responseQueue.poll(IPConnection.RESPONSE_TIMEOUT, TimeUnit.MILLISECONDS);
			if(response == null) {
				throw new IPConnection.TimeoutException("Did not receive response for getValue in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(response, 4, response.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		short valueMask = IPConnection.unsignedByte(bb.get());

		semaphoreWrite.release();
		return valueMask;
	}

	/**
	 * Configures the value and direction of the specified pins. Possible directions
	 * are "i" and "o" for input and output.
	 * 
	 * If the direction is configured as output, the value is either high or low
	 * (set as true or false).
	 * 
	 * If the direction is configured as input, the value is either pull up or
	 * default (set as true or false).
	 * 
	 * For example: 
	 * 
	 * * (15, 'i', true) will set all pins of as input pull up.
	 * * (8, 'i', false) will set pin 3 of as input default (floating if nothing is connected).
	 * * (3, 'o', false) will set pins 0 and 1 as output low.
	 * * (4, 'o', true) will set pin 2 of as output high.
	 */
	public void setConfiguration(short pinMask, char direction, boolean value)  {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_SET_CONFIGURATION, (short)7);
		bb.put((byte)pinMask);
		bb.put((byte)direction);
		bb.put((byte)(value ? 1 : 0));

		ipcon.write(this, bb, FUNCTION_SET_CONFIGURATION, false);
	}

	/**
	 * Returns a value bit mask and a direction bit mask.
	 * 
	 * For example: A return value of 0b0011 and 0b0101 for
	 * direction and value means that:
	 * 
	 * * pin 0 is configured as input pull up,
	 * * pin 1 is configured as input default,
	 * * pin 2 is configured as output high
	 * * and pin 3 is are configured as output low.
	 */
	public Configuration getConfiguration() throws IPConnection.TimeoutException {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_GET_CONFIGURATION, (short)4);

		ipcon.write(this, bb, FUNCTION_GET_CONFIGURATION, true);

		byte[] response = null;
		try {
			response = responseQueue.poll(IPConnection.RESPONSE_TIMEOUT, TimeUnit.MILLISECONDS);
			if(response == null) {
				throw new IPConnection.TimeoutException("Did not receive response for getConfiguration in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(response, 4, response.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		Configuration obj = new Configuration();
		obj.directionMask = IPConnection.unsignedByte(bb.get());
		obj.valueMask = IPConnection.unsignedByte(bb.get());

		semaphoreWrite.release();
		return obj;
	}

	/**
	 * Sets the debounce period of the {@link com.tinkerforge.BrickletIO4.InterruptListener} listener in ms.
	 * 
	 * For example: If you set this value to 100, you will get the interrupt
	 * maximal every 100ms. This is necessary if something that bounces is
	 * connected to the IO-4 Bricklet, such as a button.
	 * 
	 * The default value is 100.
	 */
	public void setDebouncePeriod(long debounce)  {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_SET_DEBOUNCE_PERIOD, (short)8);
		bb.putInt((int)debounce);

		ipcon.write(this, bb, FUNCTION_SET_DEBOUNCE_PERIOD, false);
	}

	/**
	 * Returns the debounce period as set by {@link com.tinkerforge.BrickletIO4.setDebouncePeriod}.
	 */
	public long getDebouncePeriod() throws IPConnection.TimeoutException {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_GET_DEBOUNCE_PERIOD, (short)4);

		ipcon.write(this, bb, FUNCTION_GET_DEBOUNCE_PERIOD, true);

		byte[] response = null;
		try {
			response = responseQueue.poll(IPConnection.RESPONSE_TIMEOUT, TimeUnit.MILLISECONDS);
			if(response == null) {
				throw new IPConnection.TimeoutException("Did not receive response for getDebouncePeriod in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(response, 4, response.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		long debounce = IPConnection.unsignedInt(bb.getInt());

		semaphoreWrite.release();
		return debounce;
	}

	/**
	 * Sets the pins on which an interrupt is activated with a bit mask. 
	 * Interrupts are triggered on changes of the voltage level of the pin,
	 * i.e. changes from high to low and low to high.
	 * 
	 * For example: An interrupt bit mask of 9 will enable the interrupt for 
	 * pins 0 and 3.
	 * 
	 * The interrupt is delivered with the listener {@link com.tinkerforge.BrickletIO4.InterruptListener}.
	 */
	public void setInterrupt(short interruptMask)  {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_SET_INTERRUPT, (short)5);
		bb.put((byte)interruptMask);

		ipcon.write(this, bb, FUNCTION_SET_INTERRUPT, false);
	}

	/**
	 * Returns the interrupt bit mask as set by :func:`SetPortInterrupt`.
	 */
	public short getInterrupt() throws IPConnection.TimeoutException {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_GET_INTERRUPT, (short)4);

		ipcon.write(this, bb, FUNCTION_GET_INTERRUPT, true);

		byte[] response = null;
		try {
			response = responseQueue.poll(IPConnection.RESPONSE_TIMEOUT, TimeUnit.MILLISECONDS);
			if(response == null) {
				throw new IPConnection.TimeoutException("Did not receive response for getInterrupt in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(response, 4, response.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		short interruptMask = IPConnection.unsignedByte(bb.get());

		semaphoreWrite.release();
		return interruptMask;
	}

	/**
	 * Registers a listener object.
	 */
	public void addListener(Object o) {
		if(o instanceof InterruptListener) {
			listenerObjects[CALLBACK_INTERRUPT] = o;
		}
	}
}